function [fmin,x,v,u] = main(showplot)

if nargin < 1,
	showplot = 0;
end

% time steps
T = 13;
% dimensions of x,v,u
Ndim = 2;
% number of particles
Npart = 5;

% chance constraint for failure to reach target area or to hit obstacle
delta = .2;

Ts		= .5;		% discretization, unit s
utrans	= .9;		% acceleration transmission
vcons	= .9;		% velocity conservation
Ulimit	= [-1 1];	% acceleration limit, unit?
Vlimit	= [-1 1];	% velocity limit, unit m/s
dulim = Ulimit(2)*.2;

x0 = repmat([0 0]',1,Npart);
v0 = [0.0;0.0];
u0 = [0.0;0.0];

% big M
M = 10;

% iteration limit for solver
param.itlim = 250000;
param.itlim = -1;

%param.scale = 2;
%param.presol = 0;

% target box
G = [3 1 3.5 1.5];
%G = [3.5, -.5, 4.5, .5];

% earliest arrival time (particles must arrive between st and T)
st = T-2;

%{
obst = {%[1 0 2 4 2.5 0];
		%[1 0 1 1.5 2.5 1.5 2.5 0];
		[1 0 1 1.7 2.5 1.5 2.3 0.1];
		%[.5 1 2.5 2.5 3 .5];
		%[2.5 -.5 3.5 .5 4 0]%;
		[.5 -.6 .5 -.3 3.5 0 3 -.6];
		};
%}

obst={
	[1 0 1.3 1.3 2.5 1.5 2.4 0.4];% 2.4 0.4 1.9 0.1];
	%[1 0 1 1.7 2.5 1.5 2.3 0.1];
%	[1 -.5 3 .5 3 -.5];
	[.5 -.30 3.2 .50 3 -.6];
};

% number of obstacles
Nobst = size(obst,1);

% calculate number of constraints
Oc=0; for j = 1:Nobst, Oc=Oc+length(obst{j})/2; end
Nc = 10*T -7 + Npart*(T*(6 + Nobst + Oc) + 6 + Nobst - 4*st);
%Nc = 4*(T-1) + 6*T-4 + 2*Npart*T + 1 + Npart*(4*T - 4*st + 6) +...
%	Npart*(Nobst*(T+1) + T*Oc + 1);

% calculate number of variables
%Nvar =	Npart*(1 + Nobst + T*(1 + Nobst + Oc)) + ...
%		2*T*(Npart+1) + 3*(T-1);

%{
	x_t+1 = x_t + Ts*v_t + noise_t
	v_t+1 = vcons*v_t + utrans*u_t
	u_t - dulim <= u_t+1 <= u_t + dulim
%}

% number of variables (will be increased when variable indices are created)
N = 0;


% continuous variables:
[N, xp] = createIndices(N, T,Ndim,Npart);
[N, vp] = createIndices(N, T,Ndim);
[N, up] = createIndices(N, T-1,Ndim);
% control magnitude
[N, um] = createIndices(N, T-1);

% integer variables:

% arrival constraint (particle i is at the target at time t)
[N, a] = createIndices(N, T,Npart);

% for each particle, for each time step, for each obstacle
% need cell array because number of constraints for each obstacle can differ
d = cell(T,Npart,Nobst);
% collect the values of d
di = [];
for i = 1:Npart,
	for t = 1:T,
		for j = 1:Nobst,
			% each obstacle has a different number of constraints
			Nj = length(obst{j,:})/2;
			d(t,i,j) = {N+(1:Nj)};
			N = N + Nj;
			di = [di N+(1:Nj)];
		end
	end
end

[N, e] = createIndices(N, T,Npart,Nobst);
[N, g] = createIndices(N, Npart,Nobst);
[N, z] = createIndices(N, Npart);

% set all variables continuous by default
vartype = repmat('C',1,N);
% set binary variables
vartype(di) = 'I';
vartype(e) = 'I';
vartype(g) = 'I';
vartype(z) = 'I';
vartype(a) = 'I';

disp(['# continuous variables: ' num2str(sum(vartype == 'C'))]);
disp(['# binary     variables: ' num2str(sum(vartype == 'I'))]);

% bounds for variables
lb = repmat(-inf,1,N);
ub = repmat(inf,1,N);
% bounds for control variables
% acceleration in ?
lb(up) = Ulimit(1);
ub(up) = Ulimit(2);
% velocity in m/s
lb(vp) = Vlimit(1);
ub(vp) = Vlimit(2);

% binary variables
lb(di) = 0;	ub(di) = 1;
lb(e) = 0;	ub(e) = 1;
lb(g) = 0;	ub(g) = 1;
lb(z) = 0;	ub(z) = 1;
lb(a) = 0;	ub(a) = 1;

% preallocate space
A = zeros(Nc,N);
b = zeros(Nc,1);
%A = sparse(Nc,Nvar);
ctype = zeros(Nc,1);

% cost function
c = zeros(N,1);
% minimize control magnitude
% (weight determines whether it is more important to arrive earlier or min force)
c(um) = (1/Npart);
%c(um) = T;
% punish late arrival time for all particles
c(a) = repmat((1:T)./Npart,Npart,1);


% ------------------------------------------------------
tic

% row counter for A/b
p = 1;

%generate noise
noise = randn(T,Ndim,Npart)/50;
%noise = randn(100,Ndim,100)/50;
%noise = zeros(T,Ndim,Npart);
%noise(1,:,:) = randn(1,Ndim,Npart)/20;
%noise(1,:,:) = randn(1,Ndim,50)/20;
%save noise noise;
%load noise;

% control magnitude
cs =  [1 1;1 -1;-1 1;-1 -1];
for t = 1:T-1,
	for j = 1:4
		A(p,up(t,1)) = cs(j,1); A(p,up(t,2)) = cs(j,2); A(p,um(t)) = -1;
		ctype(p) = 'U'; p=p+1;
	end
end

% generate constraints (system dynamics)
for i = 1:Ndim,
	A(p,vp(1,i)) = 1; b(p) = v0(i) + u0(i);
	p=p+1;
	A(p,up(1,i)) = 1; b(p) = + u0(i) + dulim; ctype(p) = 'U';
	p=p+1;
	A(p,up(1,i)) = 1; b(p) = + u0(i) - dulim; ctype(p) = 'L';
	p=p+1;
	for t = 2:T,
		A(p,vp(t,i)) = 1; A(p,vp(t-1,i)) = -vcons; A(p,up(t-1,i)) = -utrans*Ts;
		p=p+1;
		if t <= T-1,
			A(p,up(t,i)) = 1; A(p,up(t-1,i)) = -1; b(p) = dulim; ctype(p) = 'U';
			p=p+1;
			A(p,up(t,i)) = 1; A(p,up(t-1,i)) = -1; b(p) = -dulim; ctype(p) = 'L';
			p=p+1;
		end
	end
end

for j = 1:Npart
	for i = 1:Ndim,
		A(p,xp(1,i,j)) = 1; b(p) = x0(i,j)+v0(i)+noise(1,i,j);
		p=p+1;
		for t = 2:T,
			A(p,xp(t,i,j)) = 1; A(p,xp(t-1,i,j))= -1; A(p,vp(t-1,i)) = -Ts;
			b(p) = noise(t,i,j);
			p=p+1;
		end
	end
end

% avoid all obstacles at all time steps
for i = 1:Npart,
	for j = 1:Nobst,
		for t = 1:T,
			% get indices of needed binary variables
			dl = cell2mat(d(t,i,j));
			[aa,bb] = getObstacleConstraints(cell2mat(obst(j)));
			for k = 1:length(dl),
				A(p,dl(k)) = M;
				A(p,xp(t,1,i)) = aa(k,1);
				A(p,xp(t,2,i)) = aa(k,2);
				b(p) = bb(k);
				ctype(p)='L'; p=p+1;
			end
			% obstacle j is avoided by a given particle at a given time t
			A(p,dl) = 1; b(p) = length(dl) - 1;
			A(p,e(t,i,j)) = -M;
			ctype(p)='U'; p=p+1;
		end
		% obstacle i is avoided by a given particle at all time steps
		A(p,e(1:T,i,j)) = 1; b(p) = 0;
		A(p,g(i,j)) = -M;
		ctype(p)='U'; p=p+1;
	end
	% all obstacles are avoided by a given particle at all time steps
	A(p,g(i,1:Nobst)) = 1; b(p) = 0;
	A(p,z(i)) = -M;
	ctype(p)='U'; p=p+1;
end

% constrain the probability of hitting an obstacle / arriving to delta
A(p,z) = 1; b(p) = delta*Npart;
ctype(p)='U'; p=p+1;

% arrive at target between time st and T
for j = 1:Npart
	for t = st:T,
		for i = 1:2,
			A(p,xp(t,i,j)) = 1; A(p,a(t,j)) = -M; b(p) = -M + G(i);
			ctype(p)='L'; p=p+1;
			A(p,xp(t,i,j)) = 1; A(p,a(t,j)) = M; b(p) = M + G(i+2);
			ctype(p)='U'; p=p+1;
		end
	end
	A(p,a(st:T,j)) = 1; b(p) = 1;
	A(p,z(j)) = M;
	ctype(p)='L'; p=p+1;
end

toc
% ------------------------------------------------------

p=p-1;
% all unset constrains are equality constraints
for i = 1:length(ctype), if ctype(i) == 0, ctype(i) = 'S'; end; end
ctype(length(ctype)+1:p) = repmat('S',1,p-length(ctype));
% convert to char vector
ctype = char(ctype);

tic
% solve the optimization problem
[xopt, fmin, status, extra] = glpk(c, A, b, lb, ub, ctype, vartype, 1, param);
toc

x = xopt(xp);
v = xopt(vp);
u = xopt(up);
fm = xopt(um);

% problem couldn't be solved
if status ~= 171, return; end
save lastsolution;

[foo, arrival] = max(xopt(a));
disp(['Arrived at time step ' num2str(arrival)]);

% plot results
if showplot == 1,
	plotuav(x,v,u,fm,G,obst);
end

% -----------------------------------------------------------------------

function [a,b] = getObstacleConstraints(points)

% number of points of obstacle
P = length(points)/2;

for d = 1:2,
	x(:,d) = points(d:2:end)';
	z(:,d) = (x(1:end,d) - [x(2:end,d); x(1,d)])';
end
%x, z

a = zeros(P,2);
for j = 1:P,
	if sum(z(j,:).^2) == 0, error('Invalid polygon.'); end
	
	% under-determined system of linear equations
	% choose one a to be 1 so that we don't get the simple solution [0 0]
	if z(j,1) == 0 || z(j,2) == 0,
		a(j,1) = z(j,1) == 0;
		a(j,2) = z(j,1) ~= 0;
	else
		a(j,1) = 1;
		a(j,2) = -z(j,1)/z(j,2);
	end
	% scale to unit length
	a(j,:) = a(j,:)/norm(a(j,:));
	% calculate b
	b(j) = a(j,:) * x(j,:)';
	% determine the turn of the polygon to get a point which lies outside
	turn = x(mod(j+1,P)+1,:) - x(mod(j,P)+1,:);
	% check whether a(j,:) is pointing inwards or outwards
	m(j) = a(j,:)*(x(mod(j,P)+1,:)-turn)';
	% change orientation if necessary
	if m(j) < b(j), a(j,:) = -a(j,:); b(j) = -b(j); end
	% recalculate checks
	m(j) = a(j,:)*(x(mod(j,P)+1,:)-turn)';
end
%a,b,m


function [off, indices] = createIndices(off, varargin)
dims = cell2mat(varargin);
if length(varargin) == 1,
	indices = off+(1:dims(1));
else
	indices = reshape(off+(1:prod(dims)), dims);
end
off = off + prod(dims);


% ====================================================================
% [xopt, fmin, status, extra] = glpk (c, a, b, lb, ub, ctype, vartype,
% sense, param)
%
% Solve an LP/MILP problem using the GNU GLPK library. Given three
% arguments, glpk solves the following standard LP:
% 
% min C'*x subject to A*x  <= b
%
% but may also solve problems of the form
% 
% [ min | max ] C'*x
% subject to
%   A*x [ "=" | "<=" | ">=" ] b
%   x >= LB
%   x <= UB
%
% Input arguments:
% c = A column array containing the objective function coefficients.
% 
% a = A matrix containing the constraints coefficients.
% 
% b = A column array containing the right-hand side value for each constraint
%     in the constraint matrix.
% 
% lb = An array containing the lower bound on each of the variables.  If
%      lb is not supplied, the default lower bound for the variables is
%      zero.
% 
% ub = An array containing the upper bound on each of the variables.  If
%      ub is not supplied, the default upper bound is assumed to be
%      infinite.
% 
% ctype = An array of characters containing the sense of each constraint in the
%         constraint matrix.  Each element of the array may be one of the
%         following values
%           'F' Free (unbounded) variable (the constraint is ignored).
%           'U' Variable with upper bound ( A(i,:)*x <= b(i)).
%           'S' Fixed Variable (A(i,:)*x = b(i)).
%           'L' Variable with lower bound (A(i,:)*x >= b(i)).
%           'D' Double-bounded variable (A(i,:)*x >= -b(i) and A(i,:)*x <= b(i)).
%  
% vartype = A column array containing the types of the variables.
%               'C' Continuous variable.
%               'I' Integer variable
%
% sense = If sense is 1, the problem is a minimization.  If sense is
%         -1, the problem is a maximization.  The default value is 1.
% 
% param = A structure containing the following parameters used to define the
%         behavior of solver.  Missing elements in the structure take on default
%         values, so you only need to set the elements that you wish to change
%         from the default.
% 
%         Integer parameters:
%           msglev (LPX_K_MSGLEV, default: 1) 
%                  Level of messages output by solver routines:
%                   0 - No output.
%                   1 - Error messages only.
%                   2 - Normal output.
%                   3 - Full output (includes informational messages).
% 
%           scale (LPX_K_SCALE, default: 1). Scaling option: 
%                   0 - No scaling.
%                   1 - Equilibration scaling.
%                   2 - Geometric mean scaling, then equilibration scaling.
%
%           dual (LPX_K_DUAL, default: 0). Dual simplex option:
%                   0 - Do not use the dual simplex.
%                   1 - If initial basic solution is dual feasible, use
%                       the dual simplex.
%
%           price (LP_K_PRICE, default: 1). Pricing option (for both primal and dual simplex):
%                   0 - Textbook pricing.
%                   1 - Steepest edge pricing.
%   
%           round (LPX_K_ROUND, default: 0). Solution rounding option:
% 
%                   0 - Report all primal and dual values "as is".
%                   1 - Replace tiny primal and dual values by exact zero.
%
%           itlim (LPX_K_ITLIM, default: -1). Simplex iterations limit.  
%                 If this value is positive, it is decreased by one each
%                 time when one simplex iteration has been performed, and
%                 reaching zero value signals the solver to stop the search. 
%                 Negative value means no iterations limit.
% 
%           itcnt (LPX_K_OUTFRQ, default: 200). Output frequency, in iterations.  
%                 This parameter specifies how frequently the solver sends 
%                 information about the solution to the standard output.
% 
%           branch (LPX_K_BRANCH, default: 2). Branching heuristic option (for MIP only):
%                   0 - Branch on the first variable.
%                   1 - Branch on the last variable.
%                   2 - Branch using a heuristic by Driebeck and Tomlin.
%
%           btrack (LPX_K_BTRACK, default: 2). Backtracking heuristic option (for MIP only):
%                   0 - Depth first search.
%                   1 - Breadth first search.
%                   2 - Backtrack using the best projection heuristic.
% 
%           presol (LPX_K_PRESOL, default: 1). If this flag is set, the routine 
%                  lpx_simplex solves the problem using the built-in LP presolver. 
%                  Otherwise the LP presolver is not used.
%        
%           usecuts (LPX_K_USECUTS, default: 1). If this flag is set, the
%                  routine lpx_intopt generates and adds cutting planes to
%                  the MIP problem in order to improve its LP relaxation
%                  before applying the branch&bound method (Only Gomory's
%                  mixed integer cuts are implemented).
% 
%           lpsolver (default: 1) Select which solver to use.
%                    If the problem is a MIP problem this flag will be ignored.
%                       1 - Revised simplex method.
%                       2 - Interior point method.
%
%           save (default: 0). If this parameter is nonzero, save a copy of 
%                the problem problem in CPLEX LP format to the file "outpb.lp".  
%                There is currently no way to change the name of the output file.
% 
%         Real parameters:
%           relax (LPX_K_RELAX, default: 0.07). Relaxation parameter used 
%                 in the ratio test. If it is zero, the textbook ratio test 
%                 is used. If it is non-zero (should be positive), Harris'
%                 two-pass ratio test is used. In the latter case on the 
%                 first pass of the ratio test basic variables (in the case 
%                 of primal simplex) or reduced costs of non-basic variables 
%                 (in the case of dual simplex) are allowed to slightly violate 
%                 their bounds, but not more than relax*tolbnd or relax*toldj 
%                 (thus, relax is a percentage of tolbnd or toldj).
% 
%           tolbnd (LPX_K_TOLBND, default: 10e-7). Relative tolerance used 
%                  to check ifthe current basic solution is primal feasible.
%                  It is not recommended that you change this parameter 
%                  unless you have a detailed understanding of its purpose.
% 
%           toldj (LPX_K_TOLDJ, default: 10e-7). Absolute tolerance used to 
%                 check if the current basic solution is dual feasible.  It 
%                 is not recommended that you change this parameter unless 
%                 you have a detailed understanding of its purpose.
% 
%           tolpiv (LPX_K_TOLPIV, default: 10e-9). Relative tolerance used 
%                  to choose eligible pivotal elements of the simplex table.
%                  It is not recommended that you change this parameter 
%                  unless you have a detailed understanding of its purpose.
% 
%           objll (LPX_K_OBJLL, default: -DBL_MAX). Lower limit of the 
%                 objective function. If on the phase II the objective
%                 function reaches this limit and continues decreasing, the
%                 solver stops the search. This parameter is used in the 
%                 dual simplex method only.
% 
%           objul (LPX_K_OBJUL, default: +DBL_MAX). Upper limit of the 
%                 objective function. If on the phase II the objective
%                 function reaches this limit and continues increasing, 
%                 the solver stops the search. This parameter is used in 
%                 the dual simplex only.
% 
%           tmlim (LPX_K_TMLIM, default: -1.0). Searching time limit, in 
%                 seconds. If this value is positive, it is decreased each 
%                 time when one simplex iteration has been performed by the
%                 amount of time spent for the iteration, and reaching zero 
%                 value signals the solver to stop the search. Negative 
%                 value means no time limit.
% 
%           outdly (LPX_K_OUTDLY, default: 0.0). Output delay, in seconds. 
%                  This parameter specifies how long the solver should 
%                  delay sending information about the solution to the standard
%                  output. Non-positive value means no delay.
% 
%           tolint (LPX_K_TOLINT, default: 10e-5). Relative tolerance used 
%                  to check if the current basic solution is integer
%                  feasible. It is not recommended that you change this 
%                  parameter unless you have a detailed understanding of 
%                  its purpose.
% 
%           tolobj (LPX_K_TOLOBJ, default: 10e-7). Relative tolerance used 
%                  to check if the value of the objective function is not 
%                  better than in the best known integer feasible solution.  
%                  It is not recommended that you change this parameter 
%                  unless you have a detailed understanding of its purpose.
% 
% Output values:
% xopt = The optimizer (the value of the decision variables at the optimum).
%
% fopt = The optimum value of the objective function.
%
% status = Status of the optimization.
%          Simplex Method:
%               180 (LPX_OPT) Solution is optimal.
%               181 (LPX_FEAS) Solution is feasible.
%               182 (LPX_INFEAS) Solution is infeasible.
%               183 (LPX_NOFEAS) Problem has no feasible solution.
%               184 (LPX_UNBND) Problem has no unbounded solution.
%               185 (LPX_UNDEF) Solution status is undefined.
%          
%          Interior Point Method:
%               150 (LPX_T_UNDEF) The interior point method is undefined.
%               151 (LPX_T_OPT) The interior point method is optimal.
%
%          Mixed Integer Method:
%               170 (LPX_I_UNDEF) The status is undefined.
%               171 (LPX_I_OPT) The solution is integer optimal.
%               172 (LPX_I_FEAS) Solution integer feasible but its optimality has not been proven
%               173 (LPX_I_NOFEAS) No integer feasible solution.
%
%          If an error occurs, status will contain one of the following
%          codes:
%               204 (LPX_E_FAULT) Unable to start the search.
%               205 (LPX_E_OBJLL) Objective function lower limit reached.
%               206 (LPX_E_OBJUL) Objective function upper limit reached.
%               207 (LPX_E_ITLIM) Iterations limit exhausted.
%               208 (LPX_E_TMLIM) Time limit exhausted.
%               209 (LPX_E_NOFEAS) No feasible solution.
%               210 (LPX_E_INSTAB) Numerical instability.
%               211 (LPX_E_SING) Problems with basis matrix.
%               212 (LPX_E_NOCONV) No convergence (interior).
%               213 (LPX_E_NOPFS) No primal feasible solution (LP presolver).
%               214 (LPX_E_NODFS) No dual feasible solution (LP presolver).
% 
% extra = A data structure containing the following fields:
%           lambda - Dual variables.
%           redcosts - Reduced Costs.
%           time - Time (in seconds) used for solving LP/MIP problem.
%           mem - Memory (in bytes) used for solving LP/MIP problem.
% 
% Example:
% 
% c = [10, 6, 4]';
% a = [ 1, 1, 1;
%      10, 4, 5;
%       2, 2, 6];
% b = [100, 600, 300]';
% lb = [0, 0, 0]';
% ub = [];
% ctype = "UUU";
% vartype = "CCC";
% s = -1;
% 
% param.msglev = 1;
% param.itlim = 100;
% 
% [xmin, fmin, status, extra] = ...
%     glpk (c, a, b, lb, ub, ctype, vartype, s, param);
%
%
% Author: Nicolo' Giorgetti <giorgetti@ing.unisi.it>
